const system = require("../system");
const settings = require("../../config/settings");
const axios = require('axios');
const moment = require('moment');
const { json } = require("sequelize");
const uuid = require('uuid');

class APIBase {
    constructor() {
        this.execClient = system.getObject("util.execClient");
        this.userCenterAction = {
            "GetOverviewInfoByUser": "getOverviewInfoByUser",
            "GetOrderList": "getOrderList",
            "GetQcOrderList": "getQcOrderList",
            "GetOrderDetail": "getOrderDetail",
            "GetApplyListByUserAndType": "getApplyListByUserAndType",
            "GetApplyAndSolutionInfo": "getApplyAndSolutionInfo",
            "SelPrice2TX": "selPrice2TX",
            "GetOverviewInfoByUser": "getOverviewInfoByUser",
            "SubmitGoodsinfo": "submitGoodsinfo",
            "NeedSubmit": "needSubmit",
            "NeedList": "needList",
            "NeedDetail": "needDetail",
            "GetOrderListByStatus": "getOrderListByStatus",
            "GetQualificationCertificateListByUserAndType": "getQualificationCertificateListByUserAndType",
            "GetQualificationCertificateDetail": "getQualificationCertificateDetail"
        };
    }
    //-----------------------新的模式------------------开始

    async doexecMethod(gname, methodname, pobj, query, req) {
        if (req.body.Action && this.userCenterAction[req.body.Action]) {
            req.body.actionType = this.userCenterAction[req.body.Action];
            req.body.actionBody = req.body.ActionBody;
        }
        var param = {
            pobj: pobj,
            query: query
        }
        try {
            var result = await this[methodname](pobj, query, req);
            if (!result) {
                result = system.getResult(null, "请求的方法返回值为空");
            }
            result.requestId = result.requestId || uuid.v1();
            this.execClient.execLogs("reqPath:" + req.path + "执行结果", param, "brg-user-center-apibase", result, null);
            if (req.body.Action && this.userCenterAction[req.body.Action]) {
                return await this.handleTxResult(result);
            }//处理tx返回数据
            return result;
        } catch (e) {
            console.log(e.stack, "api调用出现异常，请联系管理员..........")
            var rtnerror = system.getResultFail(-200, "出现异常，error：" + e.stack);
            rtnerror.requestId = uuid.v1();
            this.execClient.execLogs("reqPath:" + req.path + "执行异常", param, "brg-user-center-apibase", null, e.stack);
            if (req.body.Action && this.userCenterAction[req.body.Action]) {
                return {
                    "Response": {
                        "Status": -200,
                        "Error": {
                            "Code": "FailedOperation",//操作失败
                            "Message": req.path + "执行异常"
                        },
                        "RequestId": rtnerror.requestId
                    }
                };
            }
            return rtnerror;
        }
    }

    async handleTxResult(result) {
        var newResult = {
            "Response": {
                // "Status": result.status,
                // "RequestId": result.requestId
                //DataSet:""
            }
        };
        var keys = Object.keys(result);
        var self = this;
        for (let k = 0; k < keys.length; k++) {
            const tKey = keys[k];
            if (tKey != "message") {
                const newTKey = await self.InitialToUpperMulti(self, tKey);
                const instanceSet = "InstanceSet";
                const element = result[tKey];
                if (element) {
                    if (tKey != "data") {
                        if (typeof element === 'string' || typeof element === "number" || element instanceof Date || !element) {
                            newResult.Response[newTKey] = element;
                        } else {
                            newResult.Response[newTKey] = JSON.stringify(element);
                        }
                    } else {
                        if (typeof element === 'string' || typeof element === "number" || element instanceof Date || !element) {
                            newResult.Response[instanceSet] = element;
                        } else if (element instanceof Array) {
                            newResult.Response[instanceSet] = [];
                            for (let h = 0; h < element.length; h++) {
                                const arrayElement = element[h];
                                if (typeof arrayElement === 'string' || typeof arrayElement === "number" || arrayElement instanceof Date || !arrayElement) {
                                    newResult.Response[instanceSet].push(arrayElement);
                                } else if (arrayElement instanceof Array) {
                                    newResult.Response[instanceSet].push(JSON.stringify(arrayElement));
                                } else {
                                    var arrayItem = {};
                                    const arrayKeys = Object.keys(arrayElement);
                                    for (let g = 0; g < arrayKeys.length; g++) {
                                        const arrayKey = arrayKeys[g];
                                        const newArrayKey = await self.InitialToUpperMulti(self, arrayKey);
                                        const tmpArrayKeyData = arrayElement[arrayKey];
                                        if (typeof tmpArrayKeyData === 'string' || typeof tmpArrayKeyData === "number" || tmpArrayKeyData instanceof Date || !tmpArrayKeyData) {
                                            arrayItem[newArrayKey] = tmpArrayKeyData;
                                        } else {
                                            arrayItem[newArrayKey] = JSON.stringify(tmpArrayKeyData);
                                        }
                                    }
                                    newResult.Response[instanceSet].push(arrayItem);
                                }
                            }
                        } else {
                            newResult.Response[instanceSet] = {};
                            const dataKeys = Object.keys(element);
                            for (let j = 0; j < dataKeys.length; j++) {
                                const dataKey = dataKeys[j];
                                const newDataKey = await self.InitialToUpperMulti(self, dataKey);
                                const dataElement = element[dataKey];
                                if (typeof dataElement === 'string' || typeof dataElement === "number" || dataElement instanceof Date || !dataElement) {
                                    newResult.Response[instanceSet][newDataKey] = dataElement;
                                } else {
                                    newResult.Response[instanceSet][newDataKey] = JSON.stringify(dataElement);
                                }
                            }
                        }
                    }//data处理                    
                } else {
                    if (tKey != "data") {
                        newResult.Response[newTKey] = "";
                    } else {
                        newResult.Response[instanceSet] = "";
                    }
                }
            }
        }
        if (result.status != 1) {
            newResult.Response["Error"] = {
                "Code": "InternalError",//内部错误
                "Message": result.message
            }
        }
        return newResult;
    }

    async InitialToUpperMulti(self, str) {
        const tmpStrList = str.split('_');
        var tmpStr = "";
        for (let i = 0; i < tmpStrList.length; i++) {
            const elementStr = tmpStrList[i];
            if (elementStr) {
                tmpStr = tmpStr + await self.InitialToUpper(elementStr);
            }
        }
        return tmpStr;
    }
    async InitialToUpper(str) {
        if (!str) {
            return str;
        }
        var resultStr = str.substring(0, 1).toUpperCase() + str.substring(1);
        return resultStr;
    }

    //-----------------------新的模式------------------结束

    /**
          * 带超时时间的post请求
          * @param {*} params 请求数据-json格式
          * @param {*} url 请求地址
          * @param {*} ContentType 请求头类型，默认application/json
          * @param {*} headData 请求头内容-json格式，如：请求头中传递token,格式：{token:"9098902q849q0434q09439"}
          * @param {*} timeOut 超时时间
          */
    async execPostByTimeOut(params, url, ContentType, headData, timeOut = 60) {
        return await this.execClient.execPostTimeOutByBusiness("api.base", params, url, ContentType, headData, timeOut);
    }

    /**
    * 返回20位业务订单号
    * @param {*} prefix 业务前缀
    */
    async getBusUid(prefix) {
        prefix = (prefix || "");
        if (prefix) {
            prefix = prefix.toUpperCase();
        }
        var prefixlength = prefix.length;
        var subLen = 8 - prefixlength;
        var uidStr = "";
        if (subLen > 0) {
            uidStr = await this.getUidInfo(subLen, 60);
        }
        var timStr = moment().format("YYYYMMDDHHmm");
        return prefix + timStr + uidStr;
    }
    /**
     * 返回指定长度的字符串
     * @param {*} len 返回长度
     * @param {*} radix 参与计算的长度，最大为62
     */
    async getUidInfo(len, radix) {
        var chars = '0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ'.split('');//长度62，到yz长度为长36
        var uuid = [], i;
        radix = radix || chars.length;
        if (len) {
            for (i = 0; i < len; i++) uuid[i] = chars[0 | Math.random() * radix];
        } else {
            var r;
            uuid[8] = uuid[13] = uuid[18] = uuid[23] = '-';
            uuid[14] = '4';
            for (i = 0; i < 36; i++) {
                if (!uuid[i]) {
                    r = 0 | Math.random() * 16;
                    uuid[i] = chars[(i == 19) ? (r & 0x3) | 0x8 : r];
                }
            }
        }
        return uuid.join('');
    }
}
module.exports = APIBase;