const system = require("../../../system");
const ServiceBase = require("../../sve.base");
const settings = require("../../../../config/settings");
class NeedInfoService extends ServiceBase {
  constructor() {
    super("need", ServiceBase.getDaoName(NeedInfoService));
    this.needSolutionDao = system.getObject("db.need.needSolutionDao");

  }
  //服务商关闭需求
  async closeNeed(pobj) {
    if (!pobj || !pobj.actionBody) {
      return system.getResultFail(-100, "参数错误");
    }
    var ab = pobj.actionBody;
    if (!ab.needNum) {
      return system.getResultFail(-101, "需求编码不能为空");
    }
    if (!ab.note) {
      return system.getResultFail(-102, "关闭原因不能为空");
    }
    var needinfo = await this.dao.model.findOne({
      where: { need_num: ab.needNum },
      raw: true
    });
    if (!needinfo || !needinfo.id) {
      return system.getResultFail(-300, "未知需求");
    }
    if (!needinfo.status || needinfo.status == 4 || needinfo.status == 5) {
      return system.getResultFail(-301, "操作失败，需求状态错误");
    }
    var self = this;
    return await self.db.transaction(async function (t) {
      await self.dao.update({ id: needinfo.id, status: 5 }, t);
      var needsolution = await self.needSolutionDao.model.findOne({
        where: { need_num: ab.needNum },
        raw: true
      });
      if (needsolution && needsolution.id) {
        await sqlf.needSolutionDao.update({ id: needsolution.id, refusal_notes: ab.note }, t);
      }
      return system.getResultSuccess();
    })

  }

  /**
   * 
   * @param {*} actionBody 
   */
  //创建需求
  async needSubmit(actionBody) {
    if (!actionBody.contactsMoblie) {
      return system.getResultFail(-102, "contactsMoblie cannot be empty");
    }
    if (!actionBody.regionId) {
      return system.getResultFail(-102, "regionId cannot be empty");
    }
    if (!actionBody.regionName) {
      return system.getResultFail(-102, "regionName cannot be empty");
    }
    if (!actionBody.consultType) {
      return system.getResultFail(-102, "consultType cannot be empty");
    }
    if (!actionBody.consultTypeName) {
      return system.getResultFail(-102, "consultTypeName cannot be empty");
    }
    if (!actionBody.userId) {
      return system.getResultFail(-102, "userId cannot be empty");
    }
    if (!actionBody.verificationCode) {
      return system.getResultFail(-102, "verificationCode cannot be empty");
    }
    try {
      var scobj = {
        "region_id": actionBody.regionId,
        "regionName": actionBody.regionName,
        "path_code": actionBody.consultType
      }
      var serviceinfo = await this.dao.findServiceByProductType(scobj);
      if (serviceinfo.status < 0) {
        return system.getResultFail(-203, "servicer is empty");
      }
      actionBody.servicerCode = serviceinfo.data.servicer_code;
      actionBody.servicerName = serviceinfo.data.servicer_name;
      //需求创建
      var need_num = await this.getBusUid("n");
      var datavalue = {
        need_num: need_num,
        user_id: actionBody.userId,
        contacts_moblie: actionBody.contactsMoblie,
        contacts_name: actionBody.contactsName || "",
        region_id: actionBody.regionId,
        region_name: actionBody.regionName,
        consult_type: actionBody.consultType,
        consult_type_name: actionBody.consultTypeName,
        notes: actionBody.notes,
        servicer_code: actionBody.servicerCode,
        servicer_name: actionBody.servicerName,
        status: 1
      }
      if (actionBody.companyName) {
        datavalue.need_info = {
          companyName: actionBody.companyName
        };
      }
      var newneedinfo = await this.create(datavalue);
      if (!newneedinfo) {
        return system.getResultFail(-310, "newneedinfo cannot be created");
      }

      actionBody.needNum = need_num;
      //生产者------需求推送

      var pushobj = {
        "actionType": "needSubmit",// Y 功能名称
        "actionBody": {
          notifyUrl: "http://192.168.1.113:4011/api/action/need/springBoard",
          actionType: "needSubmit",
          pushUrl: settings.apiconfig.deliveryUrl() + "/entService/consultation/springBoard",
          messageBody: actionBody,
          identifyCode: "needSubmit"
        }
      };
      // this.execPostByTimeOut(pushobj, settings.apiconfig.opPushUrl());

      return system.getResultSuccess();

    } catch (e) {
      console.log(e)
      return system.getResultFail(-510, e)
    }
  }

  //需求提交回调
  async notifyNeedSubmit(actionBody) {
    if (!actionBody.needNum) {
      return system.getResultFail(-102, "needNum cannot be empty");
    }
    var needinfo = await this.findOne({ need_num: needNum });
    if (!needinfo) {
      return system.getResultFail(-102, "needinfo is empty");
    }
    needinfo.DataValue.status = 2;
    try {
      await this.update(needinfo.DataValue);
      return system.getResultSuccess();
    } catch (error) {

    }
  }

  async needList(searchObj) {
    if (!searchObj.userId) {
      return system.getResultFail(-102, "userId cannot be empty");
    }
    var sort = [["created_at", 'desc']];
    if (searchObj.sort) {
      if (searchObj.sort.key == "createdAt") {
        searchObj.sort.key = "created_at";
      }
      sort = [[searchObj.sort.key, searchObj.sort.value]]
    }
    var sobj = {
      search: {
        user_id: searchObj.userId
      },
      orderInfo: sort,
      pageInfo: {
        pageNo: Number(searchObj.pageNo) || 1,
        pageSize: Number(searchObj.pageSize) || 10
      }
    }
    if (searchObj.consultType) {
      sobj.search.consult_type = searchObj.consultType;
    }
    if (searchObj.status) {
      sobj.search.status = searchObj.status;
    }
    var r = await this.dao.findAndCountAll(sobj);
    return system.getResultSuccess(r);
  }

  async needDetail(searchObj) {
    if (!searchObj.needNum) {
      return system.getResultFail(-102, "needNum cannot be empty");
    }
    var sobj = {
      need_num: searchObj.needNum
    }
    var needInfo = await this.findOne(sobj);
    if (!needInfo) {
      return system.getResultFail(-102, "needInfo is empty");
    }
    var solutionInfo = await this.needSolutionDao.findOne(sobj);
    if (!solutionInfo) {
      return system.getResultFail(-102, "solutionInfo is empty");
    }
    var rtn = {
      needInfo: needInfo.dataValues,
      solutionInfo: solutionInfo.dataValues
    }
    return system.getResultSuccess(rtn);
  }



}
module.exports = NeedInfoService;
