const system = require("../../../system");
const ServiceBase = require("../../sve.base");
const settings = require("../../../../config/settings");
class NeedSolutionService extends ServiceBase {
  constructor() {
    super("need", ServiceBase.getDaoName(NeedSolutionService));
    this.needInfoDao = system.getObject("db.need.needInfoDao");
  }
  //服务商提交/修改方案
  async submitSolution(pobj) {
    if (!pobj || !pobj.actionBody) {
      return system.getResultFail(-100, "参数错误");
    }
    var ab = pobj.actionBody;
    if (!ab.needNum) {
      return system.getResultFail(-101, "需求号不能为空");
    }
    if (!ab.solutionContent) {
      return system.getResultFail(-102, "方案内容不能为空");
    }
    if (!ab.solutionContent.businessType) {
      return system.getResultFail(-103, "方案内容有误,商机类型不能为空");
    }

    var needinfo = await this.needInfoDao.model.findOne({
      where: { need_num: ab.needNum },
      raw: true
    });
    if (!needinfo || !needinfo.id) {
      return system.getResultFail(-300, "未知需求信息");
    }
    if (needinfo.status == 4 || needinfo.status == 5) {
      return system.getResultFail(-304, "操作失败，需求状态错误");
    }
    if (needinfo.consult_type != ab.solutionContent.businessType) {
      return system.getResultFail(-305, "商机类型与需求信息不符");
    }
    var needsolution = null;
    if (ab.solutionNum) {//方案编码，修改方案时传入，不传默认新增
      needsolution = await this.dao.model.findOne({
        where: { need_num: ab.needNum, solution_num: ab.solutionNum },
        raw: true
      });
      if (!needsolution || !needsolution.id) {
        return system.getResultFail(-301, "方案修改失败，未知方案信息");
      }
      if (needsolution.isRefusal != 1) {
        return system.getResultFail(-303, "只能修改用户驳回方案");
      }
      var new_solution_num = await this.getBusUid("NS_");
      var updateObj = {id:needsolution.id,isRefusal:0,solution_num:new_solution_num,solution_content:ab.solutionContent};
      var self = this;
      return await self.db.transaction(async function (t) {
        await self.dao.update(updateObj,t);
        await self.needInfoDao.update({id:needinfo.id,status:"3"},t);
        return system.getResultSuccess(new_solution_num);
      });

    } else {
      needsolution = await this.dao.model.findOne({
        where: { need_num: ab.needNum },
        raw: true
      });
      if (needsolution && needsolution.id) {
        return system.getResultFail(-302, "新增失败，已存在方案信息");
      }
      var solution_num = await this.getBusUid("NS_");
      var createObj = {
        solution_num:solution_num,
        need_num:ab.needNum,
        user_id:needinfo.user_id,
        user_name:needinfo.user_name,
        solution_content:ab.solutionContent
      };
      var self = this;
      return await self.db.transaction(async function (t) {
        await self.needInfoDao.update({ id: needinfo.id, status: "3" }, t);
        await self.dao.create(createObj, t);
        return system.getResultSuccess(solution_num);
      });

    }
  }

  //驳回方案
  async refusalSolution(actionBody, t) {
    if (!actionBody.needNum) {
      return system.getResultFail(-102, "needNum cannot be empty");
    }
    if (!actionBody.refusalContent) {
      return system.getResultFail(-102, "refusalContent cannot be empty");
    }
    var self = this;
    return await self.db.transaction(async function (t) {
      var sobj = {
        need_num: actionBody.needNum
      }
      var needInfo = await self.needInfoDao.findOne(sobj, t);
      if (!needInfo) {
        return system.getResultFail(-102, "needInfo is empty");
      }
      var solutionInfo = await self.dao.findOne(sobj, t);
      if (!solutionInfo) {
        return system.getResultFail(-102, "solutionInfo is empty");
      }
      needInfo.dataValues.status = 2;
      solutionInfo.dataValues.is_refusal = 0;
      solutionInfo.dataValues.solution_content.refusal_content = actionBody.refusalContent
      await self.needInfoDao.update(needInfo.dataValues, t);
      await self.dao.update(solutionInfo.dataValues, t);

      //生产者------方案反馈
      var pushobj = {
        "actionType": "needClose",// Y 功能名称
        "actionBody": {
          notifyUrl: "",
          actionType: "needClose",
          pushUrl: settings.apiconfig.deliveryUrl() + "/entService/consultation/springBoard",
          messageBody: actionBody,
          identifyCode: "needClose"
        }
      };
      self.execPostByTimeOut(pushobj, settings.apiconfig.opPushUrl());
      return system.getResultSuccess();
    })
  }
}
module.exports = NeedSolutionService;
