const system = require("../../../system");
const { PDICT } = require("../../../../config/platform");
const ServiceBase = require("../../sve.base");
var settings = require("../../../../config/settings");
class OrderInfoService extends ServiceBase {
  constructor() {
    super("order", ServiceBase.getDaoName(OrderInfoService));
    this.orderDeliveryDao = system.getObject("db.order.orderDeliveryDao");
    this.applyInfoDao = system.getObject("db.order.applyInfoDao");
    this.orderStatementDao = system.getObject("db.order.orderStatementDao");
    this.orderProductDao = system.getObject("db.order.orderProductDao");
    this.txPushLogDao = system.getObject("db.common.txPushLogDao");
  }
  //订单详情（客户查看订单进度）
  async getOrderDetail(pobj) {
    if (!pobj || !pobj.actionBody) {
      return system.getResultFail(-100, "参数错误");
    }
    var ab = pobj.actionBody;
    if (!ab.OrderNum) {
      return system.getResultFail(-101, "订单号不能为空");
    }
    if (!ab.UserId) {
      return system.getResultFail(-102, "用户id不能为空");
    }
    var queryParam = {
      where: { order_num: ab.OrderNum, user_id: ab.UserId },
      attributes: [
        "order_num",
        "tx_orders_num",   //腾讯主订单号
        "need_num",   //需求号
        "user_name",   //用户名称
        "quantity",   //订单数量
        "total_sum",   //订单总额
        "discount_amount",  //折扣金额
        "pay_total_sum",  //付订单总额
        "refund_sum",   //退款总额
        "order_status",  //订单状态1: 已付款, 2: 部分已退款, 3: 已退款,4: 已作废
        "notes"   //备注
        , "created_at", "updated_at"
      ],
      raw: true
    };
    var orderinfo = await this.dao.model.findOne(queryParam);
    if (!orderinfo || !orderinfo.order_num) {
      return system.getResultFail(-300, "未知订单信息");
    }
    var deliveryParams = {
      where: { order_num: ab.OrderNum },
      attributes: ["deliver_content", "created_at", "updated_at", "delivery_status", "delivery_status_name"],
      raw: true
    };
    var orderdelivery = await this.orderDeliveryDao.model.findOne(deliveryParams);
    orderinfo["orderdeliveryinfo"] = orderdelivery;
    return system.getResultSuccess(orderinfo);
  }
  //获取订单列表
  async getOrderList(pobj) {
    if (!pobj || !pobj.actionBody) {
      return system.getResultFail(-100, "参数错误");
    }
    var ab = pobj.actionBody;
    if (!ab.UserId) {
      return system.getResultFail(-102, "用户id不能为空");
    }
    if (!ab.ProductType) {
      return system.getResultFail(-103, "产品类型不能为空");
    }
    var result = await this.dao.getOrderList(ab);
    return result;
  }

  //获取资质证照订单列表
  async getQcOrderList(pobj) {
    if (!pobj || !pobj.actionBody) {
      return system.getResultFail(-100, "参数错误");
    }
    var ab = pobj.actionBody;
    if (!ab.UserId) {
      return system.getResultFail(-102, "用户id不能为空");
    }
    var result = await this.dao.getQcOrderList(ab);
    return result;
  }

  //订单提交
  async orderSubmit(pobj) {
    var actionBody = pobj.actionBody;
    if (!actionBody.userId) {
      return system.getResultFail(-101, "user is empty");
    }
    if (!actionBody.product) {
      return system.getResultFail(-101, "user is empty");
    }
    if (!actionBody.productSnapshotlist) {
      return system.getResultFail(-101, "txproduct is empty");
    }
    for (var i = 0; i < productSnapshot.length; i++) {
      var i = productSnapshot[i];

    }
    var ordercreate = await this.orderCreate(pobj);
    if (ordercreate.status < 0) {
      return system.getResultFail(-101, "ordercreate is error");
    }
    var orderTXobj = {

    }
    var ordercreateTX = await this.post(txobj);

    return system.getResultSuccess();
  }

  //订单创建
  async orderCreate(pobj) {
    var actionBody = pobj.actionBody.form;
    if (!actionBody.userId) {
      return system.getResultFail(-101, "user is empty");
    }
    return await self.db.transaction(async function (t) {
      //创建订单
      var orderobj = {
        order_num: actionBody.orderNum,
        need_num: actionBody.needNum || "",
        user_id: actionBody.userId,
        user_name: actionBody.userName || "",
        quantity: actionBody.quantity,
        total_sum: actionBody.totalSum || 0,
        discount_amount: actionBody.discountAmount || 0,
        pay_total_sum: actionBody.payTotalSum || 0,
        refund_sum: actionBody.refundSum || 0,
        order_status: 1,
        order_status_name: "已支付",
      }
      var orderinfo = await self.create(orderobj, t);
      //创建订单产品
      var orderProductobj = {
        order_num: pobj.actionBody.orderNum,
        user_id: actionBody.userId,
        user_name: actionBody.userName || "",
        region_id: actionBody.regionId || 0,
        region_name: actionBody.regionName || "",
        product_icon: actionBody.product.productIcon || 0,
        product_type: actionBody.product.productType || 0,
        product_type_name: actionBody.product.productTypeName || "",
        order_snapshot: actionBody.orderSnapshot || {},
        product_snapshot: actionBody.productSnapshot || {},
        servicer_code: actionBody.product.servicerCode || "",
        servicer_name: actionBody.product.servicerName || "",
        // tx_order_snapshot: actionBody.txOrderSnapshot || {}
      }
      var orderProductinfo = await self.orderProductDao.create(orderProductobj, t);
      //创建订单交付单
      var orderDeliveryobj = {
        order_num: order_num,
        user_id: actionBody.userId,
        user_name: actionBody.userName || "",
        delivery_status: 1,
        delivery_status_name: "待接单"
      }
      var orderDeliveryinfo = await self.orderProductDao.create(orderDeliveryinfo, t);
      return system.getResultSuccess();
    });
  }

  async orderPayNotify(pobj) {
    if (!pobj.actionBody.interface || !pobj.actionBody.interface.para) {
      return system.getResultFail(-101, "参数错误");
    }
    // var selobj = {
    //   "version": 1,
    //   "componentName": "qcbuy",
    //   "eventId": 143371,
    //   "timestamp": Date.now(),
    //   "user": "auto",
    //   "interface": {
    //     "interfaceName": "qcloud.Deal.getDealsByNameOrBigId",
    //     "para": {
    //       "cond": {
    //         "name": pobj.actionBody.interface.para.dealName,
    //         "ownerUin": pobj.actionBody.interface.para.uin,
    //       }
    //     }
    //   },
    //   "seqId": "647ea242-f654-965d-a62a-eabe0289d954",
    //   "spanId": "https://buy.qcloud.com;61911"
    // }
    // var txorder = this.execPostByTimeOut(selobj, "http://trade.sandbox.com/interfaces/interface.php");
    var txorder = {
      "version": 1,
      "componentName": "trade",
      "timestamp": 1506343680,
      "eventId": 642681313,
      "returnValue": 0,
      "returnCode": 0,
      "returnMessage": "ok",
      "data": {
        "totalNum": "1",
        "deals": [
          {
            "dealId": "6138696",
            "dealName": "20170829128463",
            "bigDealId": "20170829128464",
            "goodsCategoryId": "75",
            "goodsDetail": {
              "action": "firstyear",
              "years": 1,
              "timeSpan": 1,
              "timeUnit": "y",
              "regionId": 1,
              "zoneId": "",
              "email": "whoisprotect@dnspod.cn",
              "productInfo": [
                {
                  "name": "域名",
                  "value": "lipinhua.com"
                }
              ],
              "formInfo": {
                "addressType": true,
                "businessScope": "演出及经纪业务；组织文化艺术活动；演出票务代理；字画、工艺美术品（金饰品除外）、旅游纪念品的销售；演出器材的销售、租赁；艺术装饰；艺术品展示； 舞台美术、工艺美术品、包装装璜设计、制作。",
                "businessType": "公司注册",
                "companyName": "breijinhg",
                "companyProperties": "有限责任公司",
                "contactsName": "111111",
                "contactsPhone": "18738988150",
                "engagedIndustry": "文化传媒类",
                "fullAddress": "",
                "isWhether": true,
                "memoInfo": "11111111111111111111",
                "serviceArea": "北京",
                "serviceCode": "110100",
                "taxpayerType": "小规模纳税人"
              },
              "product": {
                "productIcon": null,
                "productType": "/ic/gsreg/",
                "productTypeName": "/工商服务/公司注册/",
                "servicerCode": "GSB",
                "servicerName": "公司宝"
              },
              "tpl_id": 1,
              "service": "DR_CLUB_NEW",
              "resourceId": "",
              "domain": "suiyi512.club"
            },
            "ownerUin": "100001050256",
            "platform": "1",
            "regionId": "1",
            "appId": "1254105646",
            "goodsNum": "1",
            "goodsPrice": {
              "activitySpecifiedPrice": 1,
              "price": 13800,
              "totalCost": 13800,
              "policy": 7,
              "realTotalCost": 1000,
              "policyDetail": {
                "total": 7,
                "user": 100,
                "common": 7
              },
              "timeSpan": 1,
              "timeUnit": "y",
              "goodsNum": 1,
              "unitPrice": 13800,
              "formula": "新配置价格10.8 - 老配置单价5.4"
            },
            "creater": "100001050256",
            "creatTime": "2017-08-29 20:59:14",
            "overdueTime": "2017-09-13 20:59:14",
            "payer": "100001050256",
            "billId": "20170829030000071201512598909616",
            "endTime": "2017-08-29 21:01:29",
            "cartRecId": "0",
            "payStartTime": "2017-08-29 20:59:15",
            "payEndTime": "2017-08-29 20:59:15",
            "taskStartTime": "2017-08-29 21:01:04",
            "taskEndTime": "2017-08-29 21:01:29",
            "taskDetail": {
              "flowId": 345420,
              "resourceIds": [
                374594
              ]
            },
            "status": "4",
            "reserved": "0",
            "retry": "0",
            "reportFlag": "0",
            "payMode": "1",
            "voucherDecline": null,
            "voucherId": null,
            "projectId": "0",
            "payerMode": "0",
            "agentPayer": null,
            "dealType": "0",
            "activityId": "1986",
            "phone": "",
            "ticket": "",
            "goodsExpiredTime": null,
            "updateTime": "2017-08-29 21:01:29",
            "zoneId": "0",
            "providerOwnerUin": ""
          }
        ]
      }
    }
    if (txorder.status < 0) {
      return system.getResultFail(-101, "post is error");
    }
    if (txorder.data.returnValue < 0) {
      return system.getResultFail(-101, "txorder is error");
    }
    var txorderdetail = txorder.data.deals[0];
    console.log(txorderdetail.goodsDetail,"....txorderdetail.goodsDetail.....1");
    var self = this;
    var corder = await self.db.transaction(async function (t) {
      var oldorder = await self.findOne({})
      //创建订单
      var txorderdetail = txorder.data.deals[0];
      console.log(txorderdetail.goodsDetail,"....txorderdetail.goodsDetail...2");
      var orderobj = {
        tx_orders_num: pobj.actionBody.interface.para.bigDealId,
        order_num: pobj.actionBody.interface.para.dealName,
        need_num: txorderdetail.goodsDetail.formInfo.needNum || "",
        user_id: pobj.actionBody.interface.para.uin,
        user_name: "",
        quantity: txorderdetail.goodsNum || 1,
        total_sum: txorderdetail.goodsPrice.totalCost || 0,
        discount_amount: Number(txorderdetail.goodsPrice.totalCost) - Number(txorderdetail.goodsPrice.realTotalCost),
        pay_total_sum: txorderdetail.goodsPrice.realTotalCost || 0,
        order_status: 1,
        order_status_name: "已支付",
      }
      var orderinfo = await self.create(orderobj, t);

      //创建订单产品
      var orderProductobj = {
        tx_orders_num: pobj.actionBody.interface.para.bigDealId,
        order_num: pobj.actionBody.interface.para.dealName,
        user_id: pobj.actionBody.interface.para.uin,
        region_id: txorderdetail.goodsDetail.formInfo.serviceCode || 0,
        region_name: txorderdetail.goodsDetail.formInfo.serviceArea || "",
        product_icon: txorderdetail.goodsDetail.product.productIcon || 0,
        product_type: txorderdetail.goodsDetail.product.productType || 0,
        product_type_name: txorderdetail.goodsDetail.product.productTypeName || "",
        order_snapshot: txorderdetail.goodsDetail.formInfo || {},
        servicer_code: txorderdetail.goodsDetail.product.servicerCode || "",
        servicer_name: txorderdetail.goodsDetail.product.servicerName || "",
        tx_order_snapshot: txorderdetail,
        created_at: new Date(txorderdetail.payEndTime)
      }
      var orderProductinfo = await self.orderProductDao.entTimeCreate(orderProductobj, t);
      //创建订单交付单
      var orderDeliveryobj = {
        tx_orders_num: pobj.actionBody.interface.para.bigDealId,
        order_num: pobj.actionBody.interface.para.dealName,
        user_id: pobj.actionBody.interface.para.uin,
        delivery_status: 1,
        delivery_status_name: "待接单"
      }
      var orderDeliveryinfo = await self.orderDeliveryDao.create(orderDeliveryobj, t);
      var orderPayobj = {
        tx_orders_num: pobj.actionBody.interface.para.bigDealId,
        order_num: pobj.actionBody.interface.para.dealName,
        user_id: pobj.actionBody.interface.para.uin,
        op_type: 1,
        op_type_name: "收",
        pay_total_sum: txorderdetail.goodsPrice.realTotalCost
      }
      await self.orderStatementDao.create(orderPayobj, t);
      var txPushLoginfo=await self.txPushLogDao.findOne({deal_name:pobj.actionBody.interface.para.dealName});
      if(!txPushLoginfo){
        return system.getResultFail(-101, "txPushLoginfo is error");
      }
      txPushLoginfo.dataValues.push_status=0;
      await self.txPushLogDao.update(txPushLoginfo.dataValues,t);
      return system.getResultSuccess();
    });
    //生产者------订单推送
    var pushobj = {
      "actionType": "orderSubmit",// Y 功能名称
      "actionBody": {
        notifyUrl: "http://192.168.1.113:4011/api/action/order/springBoard",
        actionType: "orderSubmit",
        pushUrl: settings.apiconfig.opPushUrl() + "/entService/consultation/springBoard",
        messageBody: {
          "txOrderNum": pobj.actionBody.interface.para.bigDealId,
          "orderNum": pobj.actionBody.interface.para.dealName,
          "userId": pobj.actionBody.interface.para.uin,
          "servicerCode": txorderdetail.goodsDetail.product.servicerCode,
          "servicerName": txorderdetail.goodsDetail.product.servicerName,
          "regionId": txorderdetail.goodsDetail.formInfo.serviceCode,
          "regionName": txorderdetail.goodsDetail.formInfo.serviceArea,
          "contactsName": txorderdetail.goodsDetail.formInfo.contactsName,
          "contactsMoblie": txorderdetail.goodsDetail.formInfo.contactsMoblie,
          "productType": txorderdetail.goodsDetail.product.productType,
          "order_snapshot": txorderdetail.goodsDetail.formInfo
        },
        identifyCode: "orderSubmit"
      }
    };
    this.execPostByTimeOut(pushobj, settings.apiconfig.opPushUrl());

    return corder;
  }

  //询价----调取腾讯接口
  async selPrice2TX(pobj) {
    var pid = 1003949;
    var goodsCategoryId = "101630";
    var goodsSubType = "";
    var billingcode = "";

    if (!pobj.actionBody.regionId) {
      return system.getResultFail(-101, "regionId is empty");
    }
    if (!pobj.actionBody.pathCode) {
      return system.getResultFail(-101, "pathCode is empty");
    }
    if (!pobj.actionBody.regionName) {
      return system.getResultFail(-101, "regionName is empty");
    }
    var priceName = null;
    //公司注册
    if (pobj.actionBody.pathCode == "/ic/gsreg/") {
      if (!pobj.actionBody.companyProperties) {
        return system.getResultFail(-101, "companyProperties is empty");
      }
      priceName = pobj.actionBody.companyProperties;
    }
    //云上园区注册 
    if (pobj.actionBody.pathCode == "/ic/cpreg/") {
      if (!pobj.actionBody.registeredType) {
        return system.getResultFail(-101, "registeredType is empty");
      }
      priceName = pobj.actionBody.registeredType;
    }
    //刻章服务
    if (pobj.actionBody.pathCode == "/ic/kzfw/") {
      if (!pobj.actionBody.whetherType) {
        return system.getResultFail(-101, "whetherType is empty");
      }
      priceName = pobj.actionBody.whetherType;
    }
    //银行开户 
    if (pobj.actionBody.pathCode == "/ic/bankopen/") {
      if (!pobj.actionBody.registeredType) {
        return system.getResultFail(-101, "registeredType is empty");
      }
      priceName = pobj.actionBody.whetherType;
    }
    //工商变更 
    if (pobj.actionBody.pathCode == "/ic/gschangs/") {

    }
    //
    var productstr = "p_business_registration";
    if (pobj.actionBody.pathCode == "/ic/sksq/") {
      productstr = "p_agent"
      if (!pobj.actionBody.taxpayerType) {
        return system.getResultFail(-101, "taxpayerType is empty");
      }
      priceName = pobj.actionBody.taxpayerType;
    }
    if (pobj.actionBody.pathCode.indexOf("/qcfw/") > -1) {
      productstr = "p_vat";
      if (pobj.actionBody.period) {
        priceName = pobj.actionBody.period;
      }

    }

    // if (pobj.actionBody.pathCode == "/ic/dljz/") {
    //   if (!pobj.actionBody.subjectType) {
    //     return system.getResultFail(-101, "subjectType is empty");
    //   }
    //   if (!pobj.actionBody.taxpayerType) {
    //     return system.getResultFail(-101, "taxpayerType is empty");
    //   }
    //   if()
    //   priceName = pobj.actionBody.subjectType + pobj.actionBody.taxpayerType;
    // }
    var sqlobj = {
      pathCode: pobj.actionBody.pathCode
    }
    var sql = "select product_code goodsSubType,pid,price_code billingcode from tx_product_price where product_name=:pathCode";
    if (priceName) {
      sql += " and price_name like '%" + priceName + "%';";
    }
    var txprice = await this.customQuery(sql, sqlobj);
    if (txprice && txprice.length > 0) {
      goodsSubType = txprice[0].goodsSubType;
      pid = txprice[0].pid;
      billingcode = txprice[0].billingcode;
    } else {
      return system.getResultFail(-203, "txprice is empty");
    }
    var productobj = {
      region_id: pobj.actionBody.regionId,
      path_code: pobj.actionBody.pathCode
    }
    if (pobj.actionBody.priceName) {
      productobj.price_name = pobj.actionBody.priceName;
    }
    var txproductinfo = await this.dao.selPrice2TX(productobj);
    if (txproductinfo.status < 0) {
      return system.getResultFail(-203, "txproduct is empty");
    }
    var txobj = {
      "version": "1.0",
      "componentName": "Market",
      "eventId": 4962132,
      "timestamp": 1504077981203,
      "seqId": "1212344513",
      "interface": {
        "interfaceName": "qcloud.price.getPrice",
        "para": {
          "uin": pobj.actionBody.userId,
          "ownerUin": pobj.actionBody.userId,
          "resInfo": [
            {
              "goodsCategoryId": goodsCategoryId,
              "currency": "CNY",
              "regionId": "1",
              "zoneId": "100001",
              "type": goodsSubType,
              "action": "purchase",
              "activityId": "0",
              "goodsNum": "1",
              "payMode": "1",
              "goodsDetail": {
                "timeSpan": "1",
                "timeUnit": "p",
                "pid": pid,
                "productCode": productstr,
                "goodsSubType": goodsSubType,
                "subProductCode": goodsSubType,
                "qifuzidingyi": pobj.actionBody.regionName
              }
            }
          ]
        }
      }
    }
    if (pobj.actionBody.userId) {
      txobj.interface.para.uin = pobj.actionBody.userId;
      txobj.interface.para.ownerUin = pobj.actionBody.userId;
    }
    if (billingcode) {
      txobj.interface.para.resInfo[0].goodsDetail[billingcode] = "1";
    }
    if (pobj.actionBody.regionName) {
      txobj.interface.para.resInfo[0].goodsDetail.qifuzidingyi = pobj.actionBody.regionName;
    }
    if (pobj.actionBody.timeSpan) {
      txobj.interface.para.resInfo[0].goodsDetail.timeSpan = pobj.actionBody.timeSpan;
      txobj.interface.para.resInfo[0].goodsDetail.timeUnit = pobj.actionBody.timeUnit;
    }
    var rtn = await this.execPostByTimeOut(txobj, "http://trade.sandbox.com/interfaces/interface.php");
    if (rtn.status < 0) {
      return system.getResultFail(-101, "post is error");
    }
    if (rtn.data.returnValue < 0) {
      return system.getResultFail(-101, "txinterface is error");
    }
    var data = {
      totalCost: rtn.data.data[0].totalCost,
      realTotalCost: rtn.data.data[0].realTotalCost,
      product: {
        productIcon: txproductinfo.data.product_icon,
        productType: pobj.actionBody.pathCode,
        productTypeName: txproductinfo.data.product_type_name,
        servicerCode: txproductinfo.data.servicer_code,
        servicerName: txproductinfo.data.servicer_name
      },
      productSnapshot: txobj.interface.para.resInfo[0].goodsDetail
    }
    return system.getResultSuccess(data);
  }

  async submitGoodsinfo(pobj) {
    if (pobj.actionBody.info.length < 1) {
      return system.getResultFail(-101, "info is empty");
    }
    if (!pobj.actionBody.info[0].productSnapshot) {
      return system.getResultFail(-101, "productSnapshot is empty");
    }
    if (!pobj.actionBody.info[0].product) {
      return system.getResultFail(-101, "product is empty");
    }
    if (!pobj.actionBody.info[0].showInfo) {
      return system.getResultFail(-101, "showInfo is empty");
    }
    var goodsCategoryId = 100114;
    if (pobj.actionBody.info[0].product.productType.indexOf("/ic/") < 0) {
      goodsCategoryId = 100114;
    }
    var so = {
      itemDetails: {
        raw_goodsData: [  //raw_goodsData为数组，支持同时购买多个商品
        ]
      }
    }
    var goodsinfo = {
      goodsCategoryId: goodsCategoryId, //商品ID
      regionId: 1, //地域ID
      projectId: 0, //项目ID
      zoneId: 100001, //可用区ID
      goodsNum: 1, //购买数量
      payMode: 1, //计费模式，1表示包年包月（预付费）
      // goodsDetail内容详见下方文档链接
      goodsDetail: {
        timeUnit: "m",
        timeSpan: 1,
        pid: "11546",//购买商品的pid
        productInfo: []   //产品详情，用于展示商品配置详情

      }
    };
    goodsinfo.goodsDetail = pobj.actionBody.info[0].productSnapshot;
    goodsinfo.goodsDetail.product = pobj.actionBody.info[0].product;
    goodsinfo.goodsDetail.productInfo = pobj.actionBody.info[0].showInfo;
    goodsinfo.goodsDetail.formInfo = pobj.actionBody.info[0].formInfo;
    so.itemDetails.raw_goodsData.push(goodsinfo);
    if (pobj.actionBody.info.length == 2) {
      var newgoodsinfo = JSON.parse(JSON.stringify(goodsinfo));
      newgoodsinfo.goodsDetail.product = pobj.actionBody.info[1].product;
      newgoodsinfo.goodsDetail = pobj.actionBody.info[1].productSnapshot;
      newgoodsinfo.goodsDetail.productInfo = pobj.actionBody.info[1].showInfo;
      newgoodsinfo.goodsDetail.formInfo = pobj.actionBody.info[1].formInfo;
      so.itemDetails.raw_goodsData.push(newgoodsinfo);
    }
    var robj = {
      "url": "https://buy.cloud.tencent.com/order/check",
      "param": so
    }
    return system.getResultSuccess(robj);
  }
}
module.exports = OrderInfoService;



