var system = require("../../../system");
var settings = require("../../../../config/settings");
const AppServiceBase = require("../../app.base");
class UtilsMsgSendService extends AppServiceBase {
    constructor() {
        super();
        this.redisClient = system.getObject("util.redisClient");
        this.prefix = "sendCode:";
    }
    /**
     * 发送验证码
     * @param {*} phoneNumber 手机号
     */
    async sendVerificationCode(phoneNumber) {
        var params = { phoneNumber: phoneNumber };
        var key = this.prefix + phoneNumber;
        var existSend = await this.redisClient.exists(key);
        if (existSend) {
            return system.getResult(null, "操作过于频繁，请稍后再试");
        }
        var randomInt = await this.getUidInfo(6, 10);
        params.messageBody = "验证码为" + randomInt + "（切勿将验证码告知他人），请在页面中60秒内输入完成验证";
        var sendResult = await this.sendMessageByPhone(params);
        if (sendResult.status === 1) {
            this.redisClient.setWithEx(key, randomInt, 60);
        }
        return sendResult;
    }
    /**
     * 获取验证码
     * @param {*} phoneNumber 手机号
     */
    async getVerificationCode(phoneNumber) {
        if (!phoneNumber) {
            return system.getResult(null, "手机号不能为空");
        }
        if (!(/^1[3|4|5|6|7|8|9][0-9]\d{8}$/.test(phoneNumber))) {
            return system.getResult(null, "手机号有误，请输入正确手机号");
        }
        var key = this.prefix + phoneNumber;
        return await this.redisClient.getCache(key);
    }
    /**
     * 发送短信信息
     * @param {*} params 格式：{phoneNumber:手机号,messageBody:消息内容}
     */
    async sendMessageByPhone(params) {
        if (!params) {
            return system.getResult(null, "params参数不能为空");
        }
        var keyCount = Object.keys(params).length;
        if (keyCount === 0) {
            return system.getResult(null, "params参数不能为空!");
        }
        if (!params.phoneNumber) {
            return system.getResult(null, "手机号不能为空");
        }
        if (!(/^1[3|4|5|6|7|8|9][0-9]\d{8}$/.test(params.phoneNumber))) {
            return system.getResult(null, "手机号有误，请输入正确手机号");
        }
        if (!params.messageBody) {
            return system.getResult(null, "消息主体不能为空");
        }
        params.phoneNumber = params.phoneNumber.indexOf("86+") >= 0 ? params.phoneNumber : "86+" + params.phoneNumber;
        var existSend = await this.redisClient.exists(params.phoneNumber);
        if (existSend) {
            return system.getResult(null, "操作过于频繁，请稍后再试");
        }
        var sendResult = await this.sendPhoneMsg(params);
        return sendResult;
    }
    async sendPhoneMsg(params) {
        var params = {
            "version": 1,
            "caller": "postman",
            "componentName": "MC",
            "eventId": 1376088908,
            "interface": {
                "interfaceName": "message.message.sendMsg4Unsubscribe",
                "para": {
                    "ownerUin": 798950673,
                    "themeId": 364,
                    "tplParams": {
                        "smsTpl": [
                            params.messageBody
                        ]
                    },
                    "receiver": {
                        "wechatList": [
                        ],
                        "phoneList": [
                            params.phoneNumber
                        ],
                        "emailList": [
                        ],
                        "subAccountList": [
                            ""
                        ]
                    },
                    "lang": "zh",
                    "sendChannel": 4
                }
            }
        }
        var result = await this.execPostByTimeOut(params, "http://dev.message.tencentyun.com");
        if (result.status != 1 || !result.data || !result.data.codeDesc || result.data.codeDesc != "success") {
            return system.getResult(null, "获取失败");
        }
        return system.getResultSuccess(null, "获取成功");
    }
}

module.exports = UtilsMsgSendService;